using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Collections;
using NUnit.Framework;
using  TopCoder.Rules.DependencyMap.View;

namespace TopCoder.Rules.DependencyMap.View.AccuracyTests.Enh2
{
	/// <summary>
	/// Builds different maps and test the layout.
	/// 
    /// @author cucu
    /// Copyright (c)2005, TopCoder, Inc. All rights reserved
    /// </summary>
    [TestFixture]
	public class Enh2
	{
        protected DependencyRenderer renderer = null;
        protected GraphicsHelper gh = null;
        protected Dependency a = null;
        protected Dependency b = null;
        protected Dependency c = null;
        protected Dependency d = null;
        protected Dependency e = null;
        protected Dependency f = null;
        protected Dependency g = null;
		protected BitMapRenderer bmpRenderer = null;

		protected  void Save(string test) 
		{
			bmpRenderer.Image.Save( @"../../test_files/enh2/" + test + "_result.bmp");
		}
		protected void Start(string test) 
		{
			bmpRenderer = new BitMapRenderer(PixelFormat.Format24bppRgb);
			renderer.AddDiagramRenderer(bmpRenderer);

			gh = new GraphicsHelper(new ShapeFactory(), new SVGGraphicsAdapter(@"../../test_files/enh2/" + test + "_expected.bmp",1,1), 10, 10, 10);

		}
        [SetUp]
        protected void SetUp()
        {
            renderer = new DependencyRenderer();

            a = new Dependency("A");
            b = new Dependency("B");
            c = new Dependency("C");
            d = new Dependency("D");
            e = new Dependency("E");
            f = new Dependency("F");
            g = new Dependency("G");

        }
        /// <summary>
        /// Test overriding a shape.
        /// </summary>
        [Test]
        public void TestShapeOverride()
        {
			Start("TestShapeOverride");
            a = new Dependency("A", ShapeFactory.SeenElement);
            renderer.Render(a);
            
            gh.ShapeSize =  gh.CalcShapeSize("A");
            gh.DrawShape(true, 1, 1, "A");
			Save("TestShapeOverride");
        }

        /// <summary>
        /// Test overriding the default padding and margin.
        /// </summary>
        [Test]
        public void TestPaddingAndMarginAndSize()
        {
			Start("TestPaddingAndMarginAndSize");

			renderer.LayoutManager = new FixedGridLayoutManager(37, 49, 17);
			String text = "this is a Padding and Margin test";
			a = new Dependency(text);
			renderer.Render(a);
			gh = new GraphicsHelper(new ShapeFactory(), new SVGGraphicsAdapter(@"../../test_files/enh1/TestPaddingAndMargin_expected.svg",1,1), 37, 49, 17);

            gh.ShapeSize =  gh.CalcShapeSize(text);
            gh.DrawShape(false, 1, 1, text);

			Save("TestPaddingAndMarginAndSize");
		}

        /// <summary>
        /// Test with only one dependency.
        /// </summary>
        [Test]
        public void TestOneDependency()
        {
			Start("TestOneDependency");
            renderer.Render(a);
            gh.ShapeSize =  gh.CalcShapeSize("A");

            gh.DrawShape(false, 1, 1, "A");

			Save("TestOneDependency");
		}

        /// <summary>
        /// Test A that depends on B
        ///    A
        ///    |
        ///    B
        /// </summary>
        [Test]
        public void TestOneChild()
        {
			Start("TestOneChild");
            a.AddDependant(b);
            renderer.Render(a);
            
            gh.ShapeSize =  gh.CalcShapeSize("A");
            
            gh.DrawShape(false,1,1, "A");
            gh.DrawShape(false,1,2, "B");
            gh.DrawArrow(1,1,1,2);

			Save("TestOneChild");		
		}

        /// <summary>
        /// Test A that depends on B and C
        ///    A
        ///   / \
        ///  B   C
        /// </summary>
        [Test]
        public void TestTwoChild()
        {
			Start("TestTwoChild");
            a.AddDependant(b);
            a.AddDependant(c);
            renderer.Render(a);
            gh.ShapeSize =  gh.CalcShapeSize("A");
            
            gh.DrawShape(false,2,1, "A");
            gh.DrawShape(false,1,2, "B");
            gh.DrawShape(false,3,2, "C");
            gh.DrawArrow(2,1,1,2);
            gh.DrawArrow(2,1,3,2);

			Save("TestTwoChild");
		}

        /// <summary>
        /// Test A that depends on B, C and D
        ///    A
        ///   /|\
        ///  B C D
        /// </summary>
        [Test]
        public void TestTreeChild()
        {
			Start("TestTreeChild");
			a.AddDependant(b);
            a.AddDependant(c);
            a.AddDependant(d);
            renderer.Render(a);
            gh.ShapeSize =  gh.CalcShapeSize("A");
            
            gh.DrawShape(false,2,1, "A");
            gh.DrawShape(false,1,2, "B");
            gh.DrawShape(false,2,2, "C");
            gh.DrawShape(false,3,2, "D");
            gh.DrawArrow(2,1,1,2);
            gh.DrawArrow(2,1,2,2);
            gh.DrawArrow(2,1,3,2);

			Save("TestTreeChild");
		}

        /// <summary>
        /// Test a chain of 4 dependencies.
        ///    A
        ///    |
        ///    B
        ///    |
        ///    C
        ///    |
        ///    D
        /// </summary>
        [Test]
        public void TestChain()
        {
			Start("TestChain");

            a.AddDependant(b);
            b.AddDependant(c);
            c.AddDependant(d);
            renderer.Render(a);
            gh.ShapeSize =  gh.CalcShapeSize("A");
            
            gh.DrawShape(false,1,1, "A");
            gh.DrawShape(false,1,2, "B");
            gh.DrawShape(false,1,3, "C");
            gh.DrawShape(false,1,4, "D");
            gh.DrawArrow(1,1,1,2);
            gh.DrawArrow(1,2,1,3);
            gh.DrawArrow(1,3,1,4);

			Save("TestChain");
		}
        /// <summary>
        /// Test the dependencies as the figure 2 of the specification:
        ///      A
        ///    /   \
        ///    B    E 
        ///   / \
        ///  C   D
        /// </summary>
        [Test]
        public void TestSpec1()
        {
			Start("TestSpec1");

            a.AddDependant(b);
            a.AddDependant(e);
            b.AddDependant(c);
            b.AddDependant(d);
            e.AddDependant(f);
            renderer.Render(a);
            gh.ShapeSize =  gh.CalcShapeSize("A");
            
            gh.DrawShape(false, 3, 1, "A");
            gh.DrawShape(false, 2, 2, "B");
            gh.DrawShape(false, 1, 3, "C");
            gh.DrawShape(false, 3, 3, "D");
            gh.DrawShape(false, 5, 2, "E");
            gh.DrawShape(false, 5, 3, "F");
            gh.DrawArrow(3,1,2,2);
            gh.DrawArrow(3,1,5,2);
            gh.DrawArrow(2,2,1,3);
            gh.DrawArrow(2,2,3,3);
            gh.DrawArrow(5,2,5,3);

			Save("TestSpec1");
		}
        
        /// <summary>
        /// Test the dependencies as the figure 6 of the specification:
        ///      A
        ///    /   \
        ///    B    C 
        ///  // \
        /// D E  F
        ///   |  /\
        ///   F G C
        /// </summary>
        [Test]
        public void TestSpec2()
        {
			Start("TestSpec2");
			
			a.AddDependant(b);
            a.AddDependant(c);
            b.AddDependant(d);
            b.AddDependant(e);
            b.AddDependant(f);
            e.AddDependant(f);
            f.AddDependant(g);
            f.AddDependant(c);
            renderer.Render(a);

            gh.ShapeSize =  gh.CalcShapeSize("A");
            
            gh.DrawShape(false, 4, 1, "A");
            gh.DrawShape(false, 3, 2, "B");
            gh.DrawShape(false, 7, 2, "C");
            gh.DrawShape(false, 1, 3, "D");
            gh.DrawShape(false, 2, 3, "E");
            gh.DrawShape(false, 4, 3, "F");
            gh.DrawShape(true,  2, 4, "F");
            gh.DrawShape(false, 3, 4, "G");
            gh.DrawShape(true,  5, 4, "C");
            gh.DrawArrow(4,1,3,2);
            gh.DrawArrow(4,1,7,2);
            gh.DrawArrow(3,2,1,3);
            gh.DrawArrow(3,2,2,3);
            gh.DrawArrow(3,2,4,3);
            gh.DrawArrow(2,3,2,4);
            gh.DrawArrow(4,3,3,4);
            gh.DrawArrow(4,3,5,4);

			Save("TestSpec2");
		}

	}
}
