/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view.AccuracyTests;

import java.awt.Insets;

import com.topcoder.rules.dependencymap.view.Dependency;
import com.topcoder.rules.dependencymap.view.FixedGridLayoutManager;
import com.topcoder.rules.dependencymap.view.RendererException;
import com.topcoder.rules.dependencymap.view.ShapeFactory;

/**
 * Test creating different maps for accuracy.
 *
 * @author cucu
 * @version 1.0
 */
public abstract class LayoutAbstractTests extends BaseLayoutTests {

    /**
     *  Test overriding a shape.
     */
    public void testShapeOverride() throws RendererException {
        a = new Dependency("A", ShapeFactory.SEEN_ELEMENT);
        renderer.render(a);

        gh.setShapeSizeTo("A");
        gh.drawShape(true, 1, 1, "A");

        compare("testShapeOverride", gh, diagramRenderer);
    }

    /**
     * Test drawing only one dependency
     *
     * @throws rendererException
     */
    public void testOneDependency() throws RendererException {
        renderer.render(a);

        gh.setShapeSizeTo("A");
        gh.drawShape(false, 1, 1, "A");

        compare("testOneDependency", gh, diagramRenderer);
    }

    /**
     *  Test overriding the default padding and margin and using a different text size.
     */
    public void testPaddingMarginAndSize() throws RendererException {
        String text = "this is a Padding and Margin test";
        a = new Dependency(text);
        Insets insets = new Insets(14, 14, 14, 14);
        FixedGridLayoutManager flm = new FixedGridLayoutManager();
        flm.setColPadding(37);
        flm.setRowPadding(49);
        flm.setTextMargin(14);
        renderer.setLayoutManager(flm);
        renderer.render(a);

        gh = new GraphicsHelper(new ShapeFactory(), graphics, 37, 49, insets);
        gh.setShapeSizeTo(text);
        gh.drawShape(false, 1, 1, text);
        
        compare("testPaddingMarginAndSize", gh, diagramRenderer);
    }

    /**
     *  Test A that depends on B
     *   A
     *   |
     *   B
     */
    public void testOneChild() throws RendererException {
        a.addDependant(b);
        renderer.render(a);

        gh.setShapeSizeTo("A");
        gh.drawShape(false, 1, 1, "A");
        gh.drawShape(false, 1, 2, "B");
        gh.drawArrow(1, 1, 1, 2);

        compare("testOneChild", gh, diagramRenderer);
    }

    /**
     * Test A that depends on B and C
     *    A
     *   / \
     *  B   C
     */
    public void testTwoChild() throws RendererException {
        a.addDependant(b);
        a.addDependant(c);
        renderer.render(a);

        gh.setShapeSizeTo("A");
        gh.drawShape(false,2,1, "A");
        gh.drawShape(false,1,2, "B");
        gh.drawShape(false,3,2, "C");
        gh.drawArrow(2,1,1,2);
        gh.drawArrow(2,1,3,2);
        
        compare("testTwoChild", gh, diagramRenderer);
    }

    /**
     * Test A that depends on B, C and D
     *    A
     *   /|\
     *  B C D
    */
    public void testTreeChild() throws RendererException {
        a.addDependant(b);
        a.addDependant(c);
        a.addDependant(d);
        renderer.render(a);

        gh.setShapeSizeTo("A");
        gh.drawShape(false,2,1, "A");
        gh.drawShape(false,1,2, "B");
        gh.drawShape(false,2,2, "C");
        gh.drawShape(false,3,2, "D");
        gh.drawArrow(2,1,1,2);
        gh.drawArrow(2,1,2,2);
        gh.drawArrow(2,1,3,2);
        
        compare("testTreeChild", gh, diagramRenderer);
    }

    /**
     * Test a chain of 4 dependencies.
     *    A
     *    |
     *    B
     *    |
     *    C
     *    |
     *    D
    */
    public void testChain() throws RendererException {
        a.addDependant(b);
        b.addDependant(c);
        c.addDependant(d);
        renderer.render(a);

        gh.setShapeSizeTo("A");
        gh.drawShape(false,1,1, "A");
        gh.drawShape(false,1,2, "B");
        gh.drawShape(false,1,3, "C");
        gh.drawShape(false,1,4, "D");
        gh.drawArrow(1,1,1,2);
        gh.drawArrow(1,2,1,3);
        gh.drawArrow(1,3,1,4);
        
        compare("testChain", gh, diagramRenderer);
    }
    /**
     * Test the dependencies as the figure 2 of the specification:
     *      A
     *    /   \
     *    B    E
     *   / \   |
     *  C   D  F
    */
    public void testSpec1() throws RendererException {
        a.addDependant(b);
        a.addDependant(e);
        b.addDependant(c);
        b.addDependant(d);
        e.addDependant(f);
        renderer.render(a);

        gh.setShapeSizeTo("A");
        
        if (TestsHelper.enh4) {
		    gh.drawShape(false, 2.5, 1, "A");
		    gh.drawShape(false, 2, 2, "B");
		    gh.drawShape(false, 1, 3, "C");
		    gh.drawShape(false, 3, 3, "D");
		    gh.drawShape(false, 4, 2, "E");
		    gh.drawShape(false, 4, 3, "F");
		    gh.drawArrow(2.5,1,2,2);
		    gh.drawArrow(2.5,1,4,2);
		    gh.drawArrow(2,2,1,3);
		    gh.drawArrow(2,2,3,3);
		    gh.drawArrow(4,2,4,3);
        } else {
		    gh.drawShape(false, 3, 1, "A");
		    gh.drawShape(false, 2, 2, "B");
		    gh.drawShape(false, 1, 3, "C");
		    gh.drawShape(false, 3, 3, "D");
		    gh.drawShape(false, 5, 2, "E");
		    gh.drawShape(false, 5, 3, "F");
		    gh.drawArrow(3,1,2,2);
		    gh.drawArrow(3,1,5,2);
		    gh.drawArrow(2,2,1,3);
		    gh.drawArrow(2,2,3,3);
		    gh.drawArrow(5,2,5,3);        	
        }

        
        compare("testSpec1", gh, diagramRenderer);
    }

    /**
     * Test the dependencies as the figure 6 of the specification:
     *      A
     *    /   \
     *    B    C
     *  // \
     * D E  F
     *   |  /\
     *   F G C
    */
    public void testSpec2() throws RendererException {
        a.addDependant(b);
        a.addDependant(c);
        b.addDependant(d);
        b.addDependant(e);
        b.addDependant(f);
        e.addDependant(f);
        f.addDependant(g);
        f.addDependant(c);
        renderer.render(a);

        
        gh.setShapeSizeTo("A");
        gh.drawShape(false, 3, 2, "B");
        gh.drawShape(false, 1, 3, "D");
        gh.drawShape(false, 2, 3, "E");
        gh.drawShape(false, 4, 3, "F");
        gh.drawShape(true,  2, 4, "F");
        gh.drawShape(false, 3, 4, "G");
        gh.drawShape(true,  5, 4, "C");
        gh.drawArrow(3,2,1,3);
        gh.drawArrow(3,2,2,3);
        gh.drawArrow(3,2,4,3);
        gh.drawArrow(2,3,2,4);
        gh.drawArrow(4,3,3,4);
        gh.drawArrow(4,3,5,4);
        if (TestsHelper.enh4) {
            gh.drawShape(false, 3.5, 1, "A");
            gh.drawShape(false, 6, 2, "C");
            gh.drawArrow(3.5,1,6,2);
            gh.drawArrow(3.5,1,3,2);
        } else {
            gh.drawShape(false, 4, 1, "A");
            gh.drawShape(false, 7, 2, "C");
            gh.drawArrow(4,1,7,2);
            gh.drawArrow(4,1,3,2);
        }
        
        compare("testSpec2", gh, diagramRenderer);
    }
    
}




